<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormFactoryInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\Core\Url;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintTempstoreRepository;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AddConditionForm extends FormBase {
  use AjaxFormHelperTrait;
  use TemplateAjaxFormHelperTrait;

  /**
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * @var \Drupal\entity_template\BlueprintStorageInterface
   */
  protected $blueprintStorage;

  /**
   * @var string
   */
  protected $templateId;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $template;

  /**
   * @var \Drupal\Core\Plugin\PluginFormFactoryInterface
   */
  protected $pluginFormFactory;

  /**
   * @var \Drupal\entity_template\BlueprintTempstoreRepository
   */
  protected $blueprintTempstore;

  /**
   * @var \Drupal\Core\Condition\ConditionInterface
   */
  protected $plugin;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.condition'),
      $container->get('plugin_form.factory'),
      $container->get('entity_template.blueprint_tempstore_repository')
    );
  }

  public function __construct(
    ConditionManager $condition_manager,
    PluginFormFactoryInterface $plugin_form_factory,
    BlueprintTempstoreRepository $blueprint_tempstore_repository
  ) {
    $this->conditionManager = $condition_manager;
    $this->pluginFormFactory = $plugin_form_factory;
    $this->blueprintTempstore = $blueprint_tempstore_repository;
  }

  /**
   * Returns a unique string identifying the form.
   *
   * The returned ID should be a unique string that can be a valid PHP function
   * name, since it's used in hook implementation names such as
   * hook_form_FORM_ID_alter().
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'add_condition_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    BlueprintStorageInterface $blueprint_storage = NULL,
    $template_key = '',
    $plugin_id = ''
  ) {
    $this->blueprintStorage = $blueprint_storage;
    $this->template = $blueprint_storage->getTemplate($template_key);

    if (!$this->plugin) {
      $this->plugin = $this->conditionManager->createInstance($plugin_id);
    }

    $form_state->set('gathered_contexts', $this->template->getContexts(TRUE));
    if ($plugin_form = $this->getPluginForm($this->plugin, ["add", "configure"])) {
      $form = $plugin_form->buildConfigurationForm($form, $form_state);
    }

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Add Condition'),
        '#button_type' => 'primary',
      ],
    ];
    if ($this->isAjax()) {
      $form['actions']['submit']['#ajax']['callback'] = '::ajaxSubmit';
      // @todo static::ajaxSubmit() requires data-drupal-selector to be the same
      //   between the various Ajax requests. A bug in
      //   \Drupal\Core\Form\FormBuilder prevents that from happening unless
      //   $form['#id'] is also the same. Normally, #id is set to a unique HTML
      //   ID via Html::getUniqueId(), but here we bypass that in order to work
      //   around the data-drupal-selector bug. This is okay so long as we
      //   assume that this form only ever occurs once on a page. Remove this
      //   workaround in https://www.drupal.org/node/2897377.
      $form['#id'] = Html::getId($form_state->getBuildInfo()['form_id']);
    }

    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($plugin_form = $this->getPluginForm($this->plugin, ["add", "configure"])) {
      $plugin_form->submitConfigurationForm($form, $form_state);
    }

    $this->template->addCondition($this->plugin->getConfiguration());
    $this->blueprintTempstore->set($this->blueprintStorage);

    $form_state->setRedirectUrl(
      $this->blueprintStorage->getEditTemplateUrl($this->template->getKey())
    );
  }

  /**
   * Get the plugin form for a given set of operations.
   *
   * @param \Drupal\Component\Plugin\PluginInspectionInterface $plugin
   * @param array $operations
   *
   * @return \Drupal\Core\Plugin\PluginFormInterface|NULL
   */
  protected function getPluginForm(PluginInspectionInterface $plugin, array $operations = ['configure']) {
    foreach ($operations as $operation) {
      if (
        $plugin instanceof PluginWithFormsInterface &&
        $plugin->hasFormClass($operation)
      ) {
        return $this->pluginFormFactory->createInstance(
          $plugin,
          $operation
        );
      }
      else if ($operation === "configure" && $plugin instanceof PluginFormInterface) {
        return $plugin;
      }
    }

    return NULL;
  }
}
