<?php

namespace Drupal\entity_template_ui\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormFactoryInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintTempstoreRepository;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\entity_template\TemplateComponentManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

abstract class ComponentFormBase extends FormBase {
  use AjaxFormHelperTrait;
  use TemplateAjaxFormHelperTrait;

  /**
   * @var \Drupal\entity_template\TemplateComponentManager
   */
  protected $componentManager;

  /**
   * @var \Drupal\entity_template\BlueprintStorageInterface
   */
  protected $blueprintStorage;

  /**
   * @var string
   */
  protected $templateId;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $template;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $rootTemplate;

  /**
   * @var \Drupal\Core\Plugin\PluginFormFactoryInterface
   */
  protected $pluginFormFactory;

  /**
   * @var \Drupal\entity_template\BlueprintTempstoreRepository
   */
  protected $blueprintTempstore;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface
   */
  protected $plugin;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_template.component'),
      $container->get('plugin_form.factory'),
      $container->get('entity_template.blueprint_tempstore_repository')
    );
  }

  public function __construct(
    TemplateComponentManager $component_manager,
    PluginFormFactoryInterface $plugin_form_factory,
    BlueprintTempstoreRepository $blueprint_tempstore_repository
  ) {
    $this->componentManager = $component_manager;
    $this->pluginFormFactory = $plugin_form_factory;
    $this->blueprintTempstore = $blueprint_tempstore_repository;
  }

  /**
   * {@inheritdoc}
   */
  public function doBuildForm(array $form, FormStateInterface $form_state, $op = "add") {
    $form_state->set('gathered_contexts', $this->template->getTemplateContexts(TRUE));

    if (
      ($this->plugin instanceof PluginWithFormsInterface) &&
      $this->plugin->hasFormClass($op)
    ) {
      $plugin_form = $this->pluginFormFactory->createInstance($this->plugin, $op);
      $form = $plugin_form->buildConfigurationForm($form, $form_state);
    }

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t(
          '@label Component',
          [
            '@label' => ucfirst($op),
          ]
        ),
        '#button_type' => 'primary',
      ],
    ];
    if ($this->isAjax()) {
      $form['actions']['submit']['#ajax']['callback'] = '::ajaxSubmit';
      // @todo static::ajaxSubmit() requires data-drupal-selector to be the same
      //   between the various Ajax requests. A bug in
      //   \Drupal\Core\Form\FormBuilder prevents that from happening unless
      //   $form['#id'] is also the same. Normally, #id is set to a unique HTML
      //   ID via Html::getUniqueId(), but here we bypass that in order to work
      //   around the data-drupal-selector bug. This is okay so long as we
      //   assume that this form only ever occurs once on a page. Remove this
      //   workaround in https://www.drupal.org/node/2897377.
      $form['#id'] = Html::getId($form_state->getBuildInfo()['form_id']);
    }

    return $form;
  }

  /**
   * Initialize important values like template, rootTemplate and blueprintStorage
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $storage
   * @param string $template_key
   */
  protected function initializeTemplate(BlueprintStorageInterface $storage, string $template_key, string $sub_template = '') {
    $this->blueprintStorage = $storage;
    $this->rootTemplate = $this->template = $storage->getTemplate($template_key);

    if (!empty($sub_template)) {
      $path = explode(':', $sub_template);
      while ($bit = array_shift($path)) {
        $this->template = $this->template->getComponent($bit);

        if (!$this->template || !($this->template instanceof TemplateInterface)) {
          throw new NotFoundHttpException();
        }
      }
    }
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $op = 'add') {
    if (
      ($this->plugin instanceof PluginWithFormsInterface) &&
      $this->plugin->hasFormClass($op)
    ) {
      $plugin_form = $this->pluginFormFactory->createInstance($this->plugin, $op);
      $plugin_form->submitConfigurationForm($form, $form_state);
    }
  }
}
