<?php

namespace Drupal\entity_template_ui\PluginForm;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\PluginFormBase;
use Drupal\Core\Plugin\PluginFormFactoryInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Component\SwappableComponentInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\entity_template\TemplateComponentManager;
use Drupal\entity_template_ui\Form\TemplateUIHelperTrait;
use Drupal\entity_template_ui\Plugin\EntityTemplate\Template\TemplateUIFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class TemplateConfigureFormBase extends PluginFormBase implements ContainerInjectionInterface {
  use StringTranslationTrait;
  use TemplateUIHelperTrait;

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $plugin;

  /**
   * @var \Drupal\entity_template\TemplateComponentManager
   */
  protected $componentManager;

  /**
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * @var \Drupal\Core\Plugin\PluginFormFactoryInterface
   */
  protected $pluginFormFactory;

  /**
   * @var \Drupal\entity_template_ui\Plugin\EntityTemplate\Template\TemplateUIFactoryInterface
   */
  protected $templateUiFactory;

  /**
   * Instantiates a new instance of this class.
   *
   * This is a factory method that returns a new instance of this class. The
   * factory should pass any needed dependencies into the constructor of this
   * class, but not the container itself. Every call to this method must return
   * a new instance of this class; that is, it may not implement a singleton.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container this instance should use.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_template.component'),
      $container->get('plugin_form.factory'),
      $container->get('context.handler'),
      $container->get('entity_template_ui.template_ui_factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplate(): TemplateInterface {
    return $this->plugin;
  }

  /**
   * TemplateConfigureFormBase constructor.
   *
   * @param \Drupal\entity_template\TemplateComponentManager $component_manager
   * @param \Drupal\Core\Plugin\PluginFormFactoryInterface $plugin_form_factory
   * @param \Drupal\Core\Plugin\Context\ContextHandlerInterface $context_handler
   * @param \Drupal\entity_template_ui\Plugin\EntityTemplate\Template\TemplateUIFactoryInterface $template_ui_factory
   */
  public function __construct(
    TemplateComponentManager $component_manager,
    PluginFormFactoryInterface $plugin_form_factory,
    ContextHandlerInterface $context_handler,
    TemplateUIFactoryInterface $template_ui_factory
  ) {
    $this->componentManager = $component_manager;
    $this->pluginFormFactory = $plugin_form_factory;
    $this->contextHandler = $context_handler;
    $this->templateUiFactory = $template_ui_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form_state->set('gathered_contexts', $this->plugin->getTemplateContexts(TRUE));

    if (!isset($form['#parents'])) {
      $form['#parents'] = [];
    }

    // Components
    $form['components'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    $form['components']['__add'] = [
      '#type' => 'link',
      '#title' => $this->t('Add Component'),
      '#url' => $this->getTemplateUi()
        ->addComponentUrl()
        ->mergeOptions([
          'attributes' => $this->ajaxAttributes()
        ]),
      '#weight' => -5,
      '#attributes' => [
        'class' => ['add-component-button', 'btn', 'button'],
      ],
    ];

    $form['components']['tabs'] = [
      '#type' => 'vertical_tabs',
    ];
    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface $component */
    foreach ($this->plugin->getComponents() as $k => $component) {
      $group = implode('][', array_merge($form['#parents'], ['components', 'tabs']));

      $form['components'][$k] = [
        '#type' => 'details',
        '#title' => $component->label(),
        '#description' => $component->description(),
        '#parents' => array_merge($form['#parents'], ['components', $k]),
        '#group' => $group,
      ];
      $sub_form_state = SubformState::createForSubform(
        $form['components'][$k],
        $form,
        $form_state
      );

      $template_ui = $this->getTemplateUi();
      // Add the operation buttons first to allow specific plugins to alter them.
      $form['components'][$k]['operations'] = [
        '#type' => 'container',
      ];
      $form['components'][$k]['operations']['remove'] = [
        '#type' => 'link',
        '#title' => $this->t('Remove'),
        '#url' => $template_ui
          ->removeComponentUrl($k)
          ->mergeOptions([
            'attributes' => $this->ajaxAttributes()
          ]),
        '#weight' => -5,
        '#attributes' => [
          'class' => ['remove-component-button', 'btn', 'button'],
        ],
      ];
      if ($component instanceof SwappableComponentInterface && $component->isSwappable()) {
        foreach ($component->canSwapTo() as $other_id => $label) {
          $form['components'][$k]['operations']['switch_'.$other_id] = [
            '#type' => 'link',
            '#title' => $this->t(
              'Switch to @label',
              [
                '@label' => $label,
              ]
            ),
            '#url' => $template_ui
              ->swapComponentUrl($k, $other_id)
              ->mergeOptions([
                'attributes' => $this->ajaxAttributes(),
              ]),
            '#attributes' => [
              'class' => ['remove-component-button', 'btn', 'button'],
            ],
          ];
        }
      }

      $form['components'][$k] = $this->getComponentForm($component)
        ->buildConfigurationForm($form['components'][$k], $sub_form_state);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $configuration = $this->plugin->getConfiguration();

    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface $component */
    foreach ($this->plugin->getComponents() as $k => $component) {
      $sub_form_state = SubformState::createForSubform(
        $form['components'][$k],
        $form,
        $form_state
      );

      $this->getComponentForm($component)
        ->submitConfigurationForm($form['components'][$k], $sub_form_state);
    }
    $configuration['components'] = $this->plugin->getComponents()->getConfiguration();

    $this->plugin->setConfiguration($configuration);
  }

  /**
   * Get the componetn form
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface $component
   *
   * @return \Drupal\Core\Plugin\PluginFormInterface
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  protected function getComponentForm(ComponentInterface $component) {
    if ($component instanceof  PluginWithFormsInterface) {
      return $this->pluginFormFactory->createInstance($component, 'configure');
    }

    return $component;
  }

  /**
   * Get the attributes for ajax links.
   *
   * @return array
   */
  protected function ajaxAttributes() {
    return  [
      'class' => [
        'use-ajax',
      ],
      'data-dialog-type' => 'dialog',
      'data-dialog-renderer' => 'off_canvas',
    ];
  }

}
