<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface;
use Drupal\entity_template\BlueprintStorageEntityAdaptor;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\TemplateBuilderManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class BuilderDefault
 *
 * @EntityTemplateBlueprintProvider(
 *   id = "builder",
 *   label = @Translation("Builder Default")
 * )
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider
 */
class BuilderDefault extends PluginBase implements BlueprintProviderInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\entity_template\TemplateBuilderManager
   */
  protected $builderManager;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.entity_template.builder')
    );
  }

  /**
   * BuilderDefault constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\entity_template\TemplateBuilderManager $builder_manager
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TemplateBuilderManager $builder_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->builderManager = $builder_manager;
  }

  /**
   * Get all the available blueprints for a given builder
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface $builder
   *
   * @return \Drupal\entity_template\BlueprintInterface[]
   */
  public function getAllBlueprints(BuilderInterface $builder) {
    return [
      'default' => $builder->getDefaultBlueprint(),
    ];
  }

  /**
   * Get the available blueprints for a given builder, limiting based on
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface $builder
   * @param array $parameters
   * @param \Drupal\Core\Session\AccountInterface|NULL $account
   *
   * @return \Drupal\entity_template\BlueprintInterface[]
   */
  public function getAvailableBlueprints(BuilderInterface $builder, $parameters = [], AccountInterface $account = NULL) {
    return $this->getAllBlueprints($builder);
  }

  /**
   * {@inheritdoc}
   *
   * In this instance the id is the name of the builder.
   */
  public function getBlueprintStorage($id) {
    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface $builder */
    $builder = $this->builderManager->createInstance($id);
    return $builder->getDefaultBlueprintStorage($this);
  }

  /**
   * Get the key of a given blueprint to identify it in the provider.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprintStorage
   *
   * @return string
   */
  public function getBlueprintKey(BlueprintStorageInterface $blueprint_storage) {
    return $blueprint_storage->getBuilder()->getPluginId();
  }

  /**
   * {@inheritdoc}
   */
  public function getBlueprintEditUrl(BlueprintStorageInterface $blueprint_storage) {
    list(,$builder_config_id) = explode(':', $blueprint_storage->getBuilder()->getPluginId());

    return Url::fromRoute(
      'entity.entity_template_builder.edit_default_blueprint',
      [
        'entity_template_builder' => $builder_config_id,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getBlueprintEditTemplateUrl(BlueprintStorageInterface $blueprint_storage, string $key): Url {
    return $blueprint_storage
      ->getBuilder()
      ->getDefaultBlueprintEditTemplateUrl($blueprint_storage, $key);
  }
}
