<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\BlueprintEntityStorageAdaptor;
use Drupal\entity_template\BlueprintEntityAdaptor;
use Drupal\entity_template\Plugin\EntityTemplate\Builder\BuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ConfigTemplateBlueprint
 *
 * @EntityTemplateBlueprintProvider(
 *   id = "blueprint_config",
 *   label = @Translation("Blueprint Configuration")
 * )
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider
 */
class ConfigTemplateBlueprint extends PluginBase implements BlueprintProviderInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('entity_type.manager'));
  }

  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getAllBlueprints(BuilderInterface $builder) {
    $storage = $this->entityTypeManager->getStorage('entity_template_blueprint');

    $ids = $storage->getQuery()
      ->condition('builder', $builder->getPluginId())
      ->condition('is_default', FALSE)
      ->execute();

    $blueprints = [];
    foreach ($storage->loadMultiple($ids) as $entity) {
      $blueprints[$entity->uuid()] = new BlueprintEntityAdaptor($entity);
    }
    return $blueprints;
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailableBlueprints(BuilderInterface $builder, $parameters = [], AccountInterface $account = NULL) {
    return $this->getAllBlueprints($builder);
  }

  /**
   * Get a blueprint storage.
   *
   * @param $id
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface
   */
  public function getBlueprintStorage($id) {
    return new BlueprintEntityStorageAdaptor(
      $this->entityTypeManager->getStorage('entity_template_blueprint')->load($id),
      $this
    );
  }

  /**
   * Get the key of a given blueprint to identify it in the provider.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprintStorage
   *
   * @return string
   */
  public function getBlueprintKey(BlueprintStorageInterface $blueprint_storage) {
    return $blueprint_storage->getEntity()->uuid();
  }

  /**
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   *
   * @return \Drupal\Core\Url
   */
  public function getBlueprintEditUrl(BlueprintStorageInterface $blueprint_storage) {
    /** @var \Drupal\entity_template\Entity\TemplateBlueprint $blueprint_entity */
    $blueprint_entity = $blueprint_storage->getEntity();

    return $blueprint_entity->toUrl();
  }

  /**
   * Get the edit template url for the blueprint.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   * @param string $key
   *
   * @return \Drupal\Core\Url
   */
  public function getBlueprintEditTemplateUrl(BlueprintStorageInterface $blueprint_storage, string $key): Url {
    return Url::fromRoute(
      'entity_template_ui.edit_template',
      [
        'template_blueprint_provider' => $blueprint_storage->getProvider()->getPluginId(),
        'blueprint_storage' => $blueprint_storage->getProviderKey(),
        'template_id' => $key,
      ]
    );
  }
}
