<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Builder;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface;
use Drupal\entity_template\BlueprintEntityStorageAdaptor;
use Drupal\entity_template\BlueprintEntityAdaptor;
use Drupal\entity_template\TemplateBlueprintProviderManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ConfigTemplateBuilder
 *
 * @EntityTemplateBuilder(
 *   id = "config",
 *   label = @Translation("Config"),
 *   deriver = "\Drupal\entity_template\Plugin\Derivative\ConfigTemplateBuilderDeriver"
 * )
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Builder
 */
class ConfigTemplateBuilder extends BuilderBase {

  /**
   * Get the template builder entity.
   *
   * @return \Drupal\entity_template\Entity\TemplateBuilder
   */
  protected function getTemplateBuilder() {
    return $this->entityTypeManager->getStorage('entity_template_builder')
      ->load($this->getPluginDefinition()['template_builder']);
  }

  /**
   * Get the default blueprint for this builder.
   *
   * @return \Drupal\entity_template\BlueprintInterface
   */
  public function getDefaultBlueprint() {
    $blueprint_storage = $this->entityTypeManager->getStorage('entity_template_blueprint');

    if (
      $this->getTemplateBuilder()->get('default_blueprint') &&
      ($default_blueprint = $blueprint_storage->load($this->getTemplateBuilder()->get('default_blueprint')))
    ) {
      /** @var \Drupal\entity_template\Entity\TemplateBlueprint $default_blueprint */
      $default_blueprint->set('is_default', TRUE);
      return new BlueprintEntityAdaptor($default_blueprint);
    }
    else {
      if (!$this->getTemplateBuilder()->get('default_blueprint')) {
        $this->getTemplateBuilder()->set('default_blueprint', $this->getTemplateBuilder()->id().'.default_blueprint');
        $this->getTemplateBuilder()->save();
      }

      return new BlueprintEntityAdaptor(
        $blueprint_storage->create([
          'id' => $this->getTemplateBuilder()->get('default_blueprint'),
          'builder' => $this->getPluginId(),
          'is_default' => TRUE,
        ])
      );
    }
  }

  /**
   * Get the default blueprint storage.
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface $provider
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface
   */
  public function getDefaultBlueprintStorage(BlueprintProviderInterface $provider) {
    $blueprint_storage = $this->entityTypeManager->getStorage('entity_template_blueprint');

    if (
      $this->getTemplateBuilder()->get('default_blueprint') &&
      ($default_blueprint = $blueprint_storage->load($this->getTemplateBuilder()->get('default_blueprint')))
    ) {
      /** @var \Drupal\entity_template\Entity\TemplateBlueprint $default_blueprint */
      $default_blueprint->set('is_default', TRUE);
      return new BlueprintEntityStorageAdaptor($default_blueprint, $provider);
    }
    else {
      if (!$this->getTemplateBuilder()->get('default_blueprint')) {
        $this->getTemplateBuilder()->set('default_blueprint', $this->getPluginId().'.default');
        $this->getTemplateBuilder()->save();
      }

      return new BlueprintEntityStorageAdaptor(
        $blueprint_storage->create([
          'id' => $this->getTemplateBuilder()->get('default_blueprint'),
          'builder' => $this->getPluginId(),
          'is_default' => TRUE,
        ]),
        $provider
      );
    }
  }

  /**
   * The type of state this returns
   *
   * @return string
   *   A typed data data type name, optionally with [] on the end to specify a
   *   list.
   */
  public function getReturnType() {
    return $this->getTemplateBuilder()->getReturnType();
  }

  /**
   * {@inheritdoc}
   */
  public function getAdminOperations() {
    $operations = parent::getAdminOperations();

    $entity = $this->getTemplateBuilder();
    if ($entity->access('update') && $entity->hasLinkTemplate('edit-form')) {
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'weight' => 10,
        'url' => $entity->toUrl('edit-form'),
      ];
    }
    if ($entity->access('delete') && $entity->hasLinkTemplate('delete-form')) {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'weight' => 100,
        'url' => $entity->toUrl('delete-form'),
      ];
    }

    return $operations;
  }
}
