<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Component;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class FieldComponentBase extends ComponentBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type')
    );
  }

  /**
   * FieldComponentBase constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    FieldTypePluginManagerInterface $field_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->fieldTypeManager = $field_type_manager;
  }

  /**
   * [@inheritdoc}
   */
  public function label() {
    return $this->getFieldDefinition()->getLabel();
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->getFieldDefinition()->getDescription();
  }

  /**
   * Get the field definition.
   *
   * @return \Drupal\Core\Field\FieldDefinitionInterface
   */
  public function getFieldDefinition() {
    return $this->entityFieldManager->getFieldDefinitions($this->getEntityTypeId(), $this->getBundle())[$this->getFieldName()];
  }

  /**
   * Get the field name
   *
   * @return string
   */
  protected function getFieldName() {
    return $this->pluginDefinition["field_name"];
  }

  /**
   * Get the bundle
   *
   * @return string
   */
  protected function getBundle() {
    return $this->entityTypeManager->getDefinition($this->getEntityTypeId())->hasKey('bundle') ?
      $this->pluginDefinition["bundle"] :
      $this->pluginDefinition["entity_type_id"];
  }

  /**
   * Get the entity type id.
   *
   * @return string
   */
  protected function getEntityTypeId() {
    return $this->pluginDefinition["entity_type_id"];
  }

  /**
   * Get a sample entity.
   *
   * @return \Drupal\Core\Entity\FieldableEntityInterface
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getSampleEntity() {
    $entity_type = $this->entityTypeManager->getDefinition($this->getEntityTypeId());

    if (!$entity_type->hasKey('bundle')) {
      return $this->entityTypeManager->getStorage($this->getEntityTypeId())->create([]);
    }
    else {
      return $this->entityTypeManager->getStorage($this->getEntityTypeId())->create([
        $entity_type->getKey('bundle') => $this->getBundle(),
      ]);
    }
  }
}
