<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Component;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\ComplexDataDefinitionInterface;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\Core\TypedData\MapDataDefinition;
use Drupal\entity_template\TemplateResult;
use Drupal\entity_template_ui\Form\TemplateUIHelperTrait;
use Drupal\typed_data\DataFetcherInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class FieldDataSelectComponent
 *
 * @EntityTemplateComponent(
 *   id = "field.data_select",
 *   label = "Select Value",
 *   category = "Select",
 *   deriver = "\Drupal\entity_template\Plugin\Derivative\EntityFieldStorageDeriver",
 * )
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Component
 */
class FieldDataSelectComponent extends FieldComponentBase implements TemplateContextAwareComponentInterface, SwappableComponentInterface {
  use TemplateContextAwareComponentTrait, TemplateUIHelperTrait {
    TemplateContextAwareComponentTrait::getTemplate insteadof TemplateUIHelperTrait;
  }

  /**
   * @var \Drupal\typed_data\DataFetcherInterface
   */
  protected $dataFetcher;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('typed_data.data_fetcher')
    );
  }

  /**
   * FieldDataSelectComponent constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   * @param \Drupal\typed_data\DataFetcherInterface $data_fetcher
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    FieldTypePluginManagerInterface $field_type_manager,
    DataFetcherInterface $data_fetcher
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager);

    $this->dataFetcher = $data_fetcher;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\typed_data\Exception\InvalidArgumentException
   */
  public function apply(EntityInterface $entity, TemplateResult $result) {
    $selector = $this->configuration['selector'];
    list($context, $path) = explode('.', $selector.'.', 2);

    if (empty($path)) {
      $entity->{$this->getFieldName()} = $this->template->getContextValue($context);
    }
    else {
      $path = substr($path, 0, -1);

      try {
        $value = $this->dataFetcher->fetchDataByPropertyPath(
          $this->template->getContext($context)->getContextData(),
          $path
        );

        if ($value && $value->getValue()) {
          $entity->{$this->getFieldName()} = $value->getValue();
        }
      }
      catch (MissingDataException $exception) {
        $result->addMessage($exception->getMessage());
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // @todo: Work out how to do an autocomplete thats sensible.

    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Select Data'),
      '#default_value' => isset($this->configuration['selector']) ? $this->configuration['selector'] : '',
      '#attributes' => [
        'class' => [ 'entity-template-ui-autocomplete' ],
      ],
      '#attached' => [
        'library' => [
          'entity_template_ui/entity_template_ui.autocomplete',
        ],
      ],
    ] + $this->getTemplateUi($this->template)
        ->dataSelectAutocompleteElement(
          $this->configuration['uuid']
        );

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $selector = $form_state->getValue('selector');
    list($context, $path) = explode('.', $selector.'.', 2);

    if (empty($path)) {
      $selected_data_def = $this->template
        ->getContextDefinition($context)
        ->getDataDefinition();
    }
    else {
      $path = substr($path, 0, -1);
      $selected_data_def = $this->dataFetcher->fetchDefinitionByPropertyPath(
        $this->template->getContextDefinition($context)->getDataDefinition(),
        $path
      );
    }

    if (!$this->definitionIsCompatible($selected_data_def)) {
      $form_state->setError($form['selector'], $this->t('The selected data is not compatible with this field.'));
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['selector'] = $form_state->getValue('selector');

    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * Check whether the data definition is compatible with this field.
   *
   * @param \Drupal\Core\TypedData\DataDefinitionInterface $definition
   */
  protected function definitionIsCompatible(DataDefinitionInterface $definition) {
    if ($definition->getDataType() === 'list') {
      /** @var \Drupal\Core\TypedData\ListDataDefinitionInterface $definition */
      // Check the compatability of the item definition.
      $definition = $definition->getItemDefinition();
    }

    $field_item_definition = $this->getFieldDefinition()->getItemDefinition();
    $data_types = [
      $field_item_definition->getDataType(),
    ];
    if (
      $field_item_definition instanceof ComplexDataDefinitionInterface &&
      $main_property = $field_item_definition->getMainPropertyName()
    ) {
      $data_types[] = $field_item_definition
        ->getPropertyDefinition($main_property)
        ->getDataType();
    }
    if ($this->getFieldDefinition()->getType() === 'entity_reference') {
      $data_types[] = 'entity:'.$this->getFieldDefinition()->getSetting('target_type');
    }

    return in_array($definition->getDataType(), $data_types);
  }

  /**
   * Compute whether the component is swappable.
   *
   * @return bool
   */
  public function isSwappable(): bool {
    return !empty($this->canSwapTo());
  }

  /**
   * Get a list of plugin ids that can be swapped between.
   *
   * @return array
   */
  public function canSwapTo(): array {
    $entity_type_id = $this->getEntityTypeId();
    $field_name = $this->getFieldName();

    $other = "field.widget_input:{$entity_type_id}.{$field_name}";
    if (\Drupal::service('plugin.manager.entity_template.component')->hasDefinition($other)) {
      return [
        $other => new TranslatableMarkup('Direct Input'),
      ];
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function swapConfigurationTo(ComponentInterface $component): ComponentInterface {
    return $component;
  }
}
