<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Component;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Plugin\PluginWithFormsInterface;
use Drupal\Core\Plugin\PluginWithFormsTrait;
use Drupal\entity_template\Plugin\EntityTemplate\Template\SubTemplateInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Template\SubTemplateTrait;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateComponentTrait;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;
use Drupal\entity_template\TemplateComponentManager;
use Drupal\entity_template\TemplateResult;
use Drupal\entity_template\TypeLabelTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class InlineTemplate
 *
 * @EntityTemplateComponent(
 *   id = "field.inline_template",
 *   label = "Inline Template",
 *   category = "Inline Template",
 *   field_types = "entity_reference",
 *   forms = {
 *     "configure" = "\Drupal\entity_template_ui\PluginForm\InlineTemplateConfigureForm",
 *     "add" = "\Drupal\entity_template_ui\PluginForm\InlineTemplateAddForm",
 *   },
 *   deriver = "\Drupal\entity_template\Plugin\Derivative\EntityFieldStorageDeriver",
 * )
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Component
 */
class InlineTemplate extends FieldComponentBase implements TemplateContextAwareComponentInterface, SubTemplateInterface, PluginWithFormsInterface {
  use PluginWithFormsTrait;
  use TemplateContextAwareComponentTrait;
  use TemplateComponentTrait;
  use TypeLabelTrait;
  use SubTemplateTrait;

  /**
   * @var \Drupal\entity_template\TemplateComponentManager
   */
  protected $componentManager;

  /**
   * @var \Drupal\entity_template\TemplateLazyComponentsCollection
   */
  protected $componentPluginCollection;

  /**
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('plugin.manager.entity_template.component'),
      $container->geT('entity_type.bundle.info'),
      $container->get('context.handler')
    );
  }

  /**
   * InlineTemplate constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   * @param \Drupal\entity_template\TemplateComponentManager $component_manager
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   * @param \Drupal\Core\Plugin\Context\ContextHandlerInterface $context_handler
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    FieldTypePluginManagerInterface $field_type_manager,
    TemplateComponentManager $component_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    ContextHandlerInterface $context_handler
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager);

    $this->componentManager = $component_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->contextHandler = $context_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'components' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * Apply this component to the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param \Drupal\entity_template\TemplateResult $result
   */
  public function apply(EntityInterface $entity, TemplateResult $result) {
    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $template_result = $this->execute();
    if ($generated = $template_result->getEntity()) {
      if ($this->getFieldDefinition()->getFieldStorageDefinition()->getCardinality() === 1) {
        $entity->{$this->getFieldName()} = $generated;
      }
      else {
        $entity->{$this->getFieldName()}[] = $generated;
      }
    }
  }

  /**
   * The key of this template within the blueprint.
   *
   * @return string
   */
  public function getKey() {
    return $this->configuration['uuid'];
  }

  /**
   * {@inheritdoc}
   */
  public function isSingle() {
    // Inline templates are always singular.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function applies() {
    // Inline templates always apply.
    return TRUE;
  }

  /**
   * Execute the template.
   *
   * @return \Drupal\entity_template\TemplateResult
   */
  public function execute() {
    $template_result = new TemplateResult();
    try {
      $entity = $this->createEntity();

      /** @var \Drupal\entity_template\Plugin\EntityTemplate\Component\ComponentInterface $plugin */
      foreach ($this->getComponents() as $plugin) {
        if ($plugin instanceof ContextAwarePluginInterface) {
          $this->contextHandler->applyContextMapping($plugin, $this->getContextProvidingTemplate()->getContexts());
        }

        $plugin->apply($entity, $template_result);
      }

      $template_result->setEntity($entity);
    }
    catch (\Exception $exception) {
      $template_result->addMessage($exception->getMessage());
    }

    return $template_result;
  }

  /**
   * @return null|string
   */
  protected function getReturnBundle() {
    if (!empty($this->configuration['target_entity_bundle'])) {
      return $this->configuration['target_entity_bundle'];
    }
    else {
      $selection_settings = $this->getFieldDefinition()->getSetting('handler_settings') ?: [];
      $bundle_info = $this->entityTypeBundleInfo->getBundleInfo(
        $this->getFieldDefinition()->getSetting('target_type')
      );

      if (!empty($selection_settings['target_bundles']) && count($selection_settings['target_bundles']) === 1) {
        return reset($selection_settings['target_bundles']);
      }
      else if (count($bundle_info) === 1) {
        return key($bundle_info);
      }
    }

    return NULL;
  }

  /**
   * Get the return type of this template.
   *
   * @return string
   */
  public function getReturnType() {
    $target_entity_type_id = $this->getFieldDefinition()->getSetting('target_type');
    $target_entity_type = $this->entityTypeManager->getDefinition($target_entity_type_id);

    if ($target_entity_type->hasKey('bundle')) {
      if ($bundle = $this->getReturnBundle()) {
        return "entity:{$target_entity_type_id}:{$bundle}";
      }
    }

    return "entity:{$target_entity_type_id}";
  }

  /**
   * Get the return type label.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   */
  public function getReturnTypeLabel() {
    return $this->getTypeLabel($this->getReturnType());
  }

  /**
   * Get the parent template.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  public function getParentTemplate() : ?TemplateInterface {
    return $this->template;
  }


  /**
   * Create the entity initially.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *
   * @throws \Drupal\entity_template\Exception\TemplateException;
   */
  protected function createEntity() {
    $target_type_id = $this->getFieldDefinition()->getSetting('target_type');
    $target_type = $this->entityTypeManager()->getDefinition($target_type_id);
    $storage = $this->entityTypeManager()->getStorage($target_type_id);

    $initial_values = [];
    if ($target_type->hasKey('bundle')) {
      $initial_values[$target_type->getKey('bundle')] = $this->getReturnBundle();
    }
    return $storage->create(array_filter($initial_values));
  }
}
